
/**************************************************************************************
* This file is part of the FrameworkTest project released by Wazoo Enterprises Inc.
* under the GNU Public License.
*
* Any projects using this code must attach these copyright notices along with their
* own as outlined at http://www.gnu.org
*
* Copyright 2001-2003 by Wazoo Enterprises Inc.
* To contact the auther, send a message to wazoo@wazooenterprises.com
*
****************************************************************************************/

/**
*WinMain - This is the win32 entry point to our project
*/
#define STRICT
#define WIN32_MEAN_AND_LEAN
#include <windows.h>
#include <objbase.h>
#include <tchar.h>
#include <stdio.h>
#include <string>
#include "winmain.h"

//version 1.3
//Added the following #define to reference our texture image
#define TITLE 1

static CTestApp* g_pApp = NULL;


LRESULT CALLBACK staticMsgProc( HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam )
{
	return( g_pApp->wndProc(hWnd, msg, wParam, lParam ) );
}

/**
*Name: WinMain
*Description: Win32 entry point.
*/
int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE, LPSTR, int) {

	CTestApp oApp;
	if(FAILED(oApp.initSystem(hInstance, "FrameworkTest", "FrameworkTest"))){
		return -1;
	}

	return( oApp.runSystem() );
}

/**
*
*/
CTestApp::CTestApp(){

	g_pApp = this;
	m_bActive = FALSE;
	m_pGraphicsFactory = NULL;

}

/**
*
*/
CTestApp::~CTestApp(){

}

/**
*
*
*/
HRESULT CTestApp::initSystem(HINSTANCE hInstance, std::string strAppClass, std::string strAppTitle){

	WNDCLASSEX	wndclass;
	TCHAR strTemp[MAX_PATH];

	sprintf(strTemp, "%s.log", strAppClass.c_str());
	Logger::getInstance()->openLog(strTemp);
	Logger::getInstance()->writeToLog("BEGIN Application logging");

	m_strAppClass = strAppClass;
	m_strAppTitle  = strAppTitle;

	// Set up window attributes
	wndclass.cbSize			= sizeof(wndclass);
	wndclass.style			= CS_HREDRAW | CS_VREDRAW |
							  CS_OWNDC   | CS_DBLCLKS;
	wndclass.lpfnWndProc	= staticMsgProc;
	wndclass.cbClsExtra		= 0;
	wndclass.cbWndExtra		= 0;
	wndclass.hInstance		= hInstance;
	wndclass.hIcon			= LoadIcon( NULL, IDI_APPLICATION );
	wndclass.hCursor		= LoadCursor( NULL, IDC_ARROW );
	wndclass.hbrBackground	= (HBRUSH)(COLOR_WINDOW);
	wndclass.lpszMenuName	= NULL;
	wndclass.lpszClassName	= m_strAppClass.c_str();	// Registered Class Name
	wndclass.hIconSm		= LoadIcon( NULL, IDI_APPLICATION );

	//register our new window class with the Windows OS
	if( RegisterClassEx( &wndclass ) == 0 ) {
		// Do error logic here
		Logger::getInstance()->writeToLog("Error registering Application");
		return E_FAIL;
	}

	Logger::getInstance()->writeToLog("Registered Application");

	// Create the main window
	//Version1.1
	//Modified the window parameters to just allow us to display the
	//main window as a 640x480 sized windowed app rather than a fullscreen one
	m_hWnd = CreateWindow(
		m_strAppClass.c_str(),				// Class Name
		m_strAppTitle.c_str(),			// Name Displayed on Title Bar
		WS_OVERLAPPEDWINDOW,           // Window Style...for now OVERLAPPEDWINDOW is fine..
		0,                  // x position (top left)
		0,                  // y position (top left)
		640, //x position (bottom right)
		480, //y position (bottom right)
		NULL,
		NULL,
		hInstance,
		NULL );

	Logger::getInstance()->writeToLog("Created Main Application window");

	ShowWindow(m_hWnd, SW_SHOW);       //we must first tell Windows to show our app
	UpdateWindow(m_hWnd);	              //updates the client by sending an initial WM_PAINT msg

	m_hInstance = hInstance;          //save our main application instance

	return S_OK;
}


/**
*
*/
int CTestApp::runSystem() {

	if(FAILED(createProgram())){
		destroyProgram();
		return -2;
	}

	BOOL bGotMsg;
    MSG  msg;
    PeekMessage( &msg, NULL, 0U, 0U, PM_NOREMOVE );
    while( WM_QUIT != msg.message  ) {
        // Use PeekMessage() if the app is active, so we can use idle time to
        // render the scene. Else, use GetMessage() to avoid eating CPU time.
        if( m_bActive )
            bGotMsg = PeekMessage( &msg, NULL, 0U, 0U, PM_REMOVE );
        else
            bGotMsg = GetMessage( &msg, NULL, 0U, 0U );

        if( bGotMsg ){
            // Translate and dispatch the message
            TranslateMessage( &msg );
            DispatchMessage( &msg );
        }
		else {
            // Render a frame during idle time (no messages are waiting)
			if(m_bActive){
				// Update application
				updateProgram(1.0f);

				// render the current scene
				renderProgram();
			}
        }
    }

	return( (int) msg.wParam);
}

/**
*Name: msgProc
*Description: This is our main window message handler
*/
LRESULT CALLBACK CTestApp::wndProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam )
{
	switch( uMsg )
    {
		case WM_ACTIVATE:
			//WM_ACTIVATE is sent whenever the application has the focus..
			//so if we minimize our app, we LOSE the focus, and therefore
			//shouldn't bother wasting clock cycles/graphics hardware
			//to render what we can't see!
			m_bActive = (BOOL)wParam;
		break;

		case WM_KEYDOWN:
			// Handle any non-accelerated key commands
            switch (wParam)
            {
				case VK_ESCAPE:
				//if the user hits ESC, then we'll just
				//quit from this application
				SendMessage(hWnd, WM_CLOSE, 0, 0);
				return 0;

			};
		break;

        case WM_DESTROY:
			//WM_DESTROY is what's sent by the Windows messaging system
			//when our message queue receives the WM_QUIT message from
			//our application...just cleans up everything and quits
			//the program..
			m_bActive = FALSE;
			destroyProgram();
            PostQuitMessage( 0 );
			
		break;

		default:
			return( DefWindowProc( hWnd, uMsg, wParam, lParam ) );

    }

	return 0; 
}

/**
*createProgram
*This function is responsible for initializing our
*game subsystems...ie. sound, input, graphics, etc..
*/
HRESULT CTestApp::createProgram(){

	HRESULT hr = S_OK;

	//first let's create an instance of our rendererFactory
	m_pGraphicsFactory = new rendererFactory(m_hInstance);

	Logger::getInstance()->writeToLog("Creating Renderer Interface");

	//we'll now tell it to read our settings from the
	//accompanying .ini file
	hr = m_pGraphicsFactory->initInterface("\\data\\config\\FrameworkTest.ini", m_hWnd);
	if(FAILED(hr))
	{
		Logger::getInstance()->writeToLog("Error creating Renderer Interface");
		return E_FAIL;
	}

	Logger::getInstance()->writeToLog("Successfully created Renderer Interface");

	//initialize our rendererInterface object
	rendererInterface *pInterface = NULL;

	//grab the interface that was created by the
	//rendererFactory
	pInterface = m_pGraphicsFactory->getInterface();
	if(pInterface == NULL)
	{
		return E_FAIL;
	}

	//now let's create the main graphics device!
	//due to proper class inheritance, we're able to call
	//the methods from our rendererInterface and have them
	//resolved automatically to our desired graphics API,
	//making these calls API independant!


	Logger::getInstance()->writeToLog("Attempting to create rendering device");
	//Version 1.1
	//Modified the device code to just render to a 640x480 window
	hr = pInterface->createDevice(m_hWnd,
								  640,
								  480,
								  16,
								  TRUE);
	if(FAILED(hr))
	{
		Logger::getInstance()->writeToLog("Failed to create rendering device");
		return E_FAIL;
	}

	Logger::getInstance()->writeToLog("Successfully created rendering device");

	//set our clearing color to blue
	pInterface->setClearColor(0.0f, 0.0f, 1.0f, 1.0f);


	Logger::getInstance()->writeToLog("Attempting to add a texture");
	//v1.3
	//get the pointer to our texture interface to add our texture
	//
	hr = pInterface->getTextureInterface()->addTexture("data\\textures\\title.tga", TITLE);
	if(FAILED(hr)){
		Logger::getInstance()->writeToLog("Failed to add a texture");
		return E_FAIL;
	}
	Logger::getInstance()->writeToLog("Successfully added texture");

	//v1.4 - change some of our states inside our graphics pipelines
	pInterface->setState( ZBUFFER, DISABLE );
	pInterface->setState( LIGHTING, DISABLE );
		
	//enable alpha-blending to take advantage of our alpha channel in the TGA image
	pInterface->setState( ALPHABLENDING, ENABLE );
	pInterface->setState( SRC_BLEND, SRC_ALPHA );
	pInterface->setState( DEST_BLEND, INV_SRC_ALPHA );

	Logger::getInstance()->writeToLog("Successfully initialized program");

	return S_OK;
}

/**
*destroyProgram
*This function is responsible for handling all of the
*cleanup of our subsystems..
*/
void CTestApp::destroyProgram(){

	
	//if our rendererFactory exists, blow it away!
	if(m_pGraphicsFactory)
	{
		delete m_pGraphicsFactory;
		m_pGraphicsFactory = NULL;
	}

	Logger::getInstance()->writeToLog("END Application logging");

}

/**
* updateProgram
* This function is responsible for handling the updating
* to each object in our game, as well as processing input,
*etc..
*/
void CTestApp::updateProgram(FLOAT fTimeKey){




}

////////////////////////////////////////
// renderProgram
// This function is responsible for handling the rendering
// of our graphics...depending on what's happening in the
// game
///////////////////////////////////////
void CTestApp::renderProgram(){

	rendererInterface *pInterface =  m_pGraphicsFactory->getInterface();
	if( NULL == pInterface )
		return;

	//let's begin our rendering loop
	if(SUCCEEDED(pInterface->beginRenderingScene()))
	{
		//v1.3 let our graphics pipeline know about our texture
		pInterface->getTextureInterface()->setTexture(TITLE);
			
		//V1.1
		//We want to render a simple square using our new VertexBuffer interface
		//V1.3
		//modified the parameters to support the new texture mapping vertices
		if(SUCCEEDED(pInterface->m_pVB->lockVB())){

			pInterface->m_pVB->addTriToVB(D3DXVECTOR3(-1.0f, -1.0f, -5.0f),
				1.0f, 1.0f, 1.0f, 0.7f, D3DXVECTOR2(0.0f, 0.0f));
			pInterface->m_pVB->addTriToVB(D3DXVECTOR3(-1.0f, 1.0f, -5.0f),
				1.0f, 1.0f, 1.0f, 0.7f, D3DXVECTOR2(0.0f, 1.0f));
			pInterface->m_pVB->addTriToVB(D3DXVECTOR3(1.0f, -1.0f, -5.0f),
				1.0f, 1.0f, 1.0f, 0.7f, D3DXVECTOR2(1.0f, 0.0f));


			pInterface->m_pVB->addTriToVB(D3DXVECTOR3(1.0f, -1.0f, -5.0f),
				1.0f, 1.0f, 1.0f, 0.7f, D3DXVECTOR2(1.0f, 0.0f));
			pInterface->m_pVB->addTriToVB(D3DXVECTOR3(-1.0f, 1.0f, -5.0f),
				1.0f, 1.0f, 1.0f, 0.7f, D3DXVECTOR2(0.0f, 1.0f));
			pInterface->m_pVB->addTriToVB(D3DXVECTOR3(1.0f, 1.0f, -5.0f),
				1.0f, 1.0f, 1.0f, 0.7f, D3DXVECTOR2(1.0f, 1.0f));


			pInterface->m_pVB->unlockVB();
		}

		
		//end our rendering loop, and signal the
		//graphics hardware to blast the primitives
		//out to the device
		pInterface->endRenderingScene();
	}


}
