
#ifndef __D3DRENDERER_H_
#define __D3DRENDERER_H_
/**************************************************************************************
* This file is part of the GameFramework project released by Wazoo Enterprises Inc.
* under the GNU Public License. 
*
* Any projects using this code must attach these copyright notices along with their
* own as outlined at http://www.gnu.org
*
* Copyright 2003 by Wazoo Enterprises Inc.
* To contact the auther, send a message to wazoo@wazooenterprises.com
*
****************************************************************************************/


#include "rendererInterface.h"
#include <d3d8.h>
#include <d3dx8.h>
#include <map>

inline DWORD FtoDW( FLOAT f ) { return *((DWORD*)&f); }

/**
* This is our "low level" implementation of
* our rendererInterface class in Direct3D8
* Note that it's VERY basic, but just meant to
* show a POSSIBLE implementation
*/
class D3DRenderer : public rendererInterface
{
private:
	LPDIRECT3D8 m_lpD3D;                          /** pointer to our IDirect3D8 interface */
	LPDIRECT3DDEVICE8 m_lpD3DDevice;              /** pointer to our IDirect3DDevice8 interface */
	D3DPRESENT_PARAMETERS m_d3dpp;                /** structure containing presentation parameters */
	D3DCOLOR m_backColor;                         /** structure containing our clearing color */


public:
	/**
	* Constructor
	*/
	D3DRenderer();

	/**
	* Destructor
	*/
	~D3DRenderer();

	/**
	* Method to create and initialize our hardware
	* @param HWND - our window handle
	* @param DWORD - our window width
	* @param DWORD - our window height
	* @param DWORD - our window depth
	* @param BOOL  - flag for being windowed or fullscreen
	* @return HRESULT - SUCCEEDED if we're ok, FAILED otherwise
	*/
    HRESULT createDevice(HWND, DWORD, DWORD, DWORD, BOOL);

    /**
	* Method to destroy it
	* @param void
	* @return void
	*/
    void destroyDevice();

    /**
	* Method to begin rendering primitives
	* @param void
	* @return HRESULT - SUCCEEDED if our hardware pipeline is ready, FAILED if not
	*/
    HRESULT beginRenderingScene();

    /**
	* Method to end rendering primitives
	* @param void
	* @return void
	*/
    void endRenderingScene();

    /**
	* Method to change the screen clearing color
	* @param FLOAT - red component
	* @param FLOAT - green component
	* @param FLOAT - blue component
	* @param FLOAT - alpha component
	* @return void
	*/
    void setClearColor(FLOAT, FLOAT, FLOAT, FLOAT);

	/**
	* (v1.3)
	* This method is responsible for adding a new texture to our
	* rendering engine. We won't do anything that fancy but store
	* it in a std::map object
	* @param std::string - our texture filename
	* @param const int   - the texture identifier
	* @return HRESULT    - SUCCEEDED if we created it ok, FAILED otherwise
	*/
	HRESULT addTexture(std::string strFilename, const int value);

	/**
	* (v1.3)
	* This method is responsible for letting the renderer engine
	* know which texture we want to use
	* @param const int - the texture identifier
	* @return void
	*/
	void setTexture(const int value);

	/**
	* v1.4 
	* Method to enable/disable a state within the engine
	* @param eState - which state we're trying to modify
	* @param eOp     - the operation to perform on said state
	* @return void
	*/
	void setState(const eState, const eOp);

	void setFunction(const eFunction, const eCompareOp, FLOAT fRef);

	
	

};

#endif