
#ifndef __D3DTEXTUREMANAGER_H_
#define __D3DTEXTUREMANAGER_H_
/**************************************************************************************
* This file is part of the GameFramework project released by Wazoo Enterprises Inc.
* under the GNU Public License. 
*
* Any projects using this code must attach these copyright notices along with their
* own as outlined at http://www.gnu.org
*
* Copyright 2003 by Wazoo Enterprises Inc.
* To contact the auther, send a message to wazoo@wazooenterprises.com
*
****************************************************************************************/



#include "textureManagerInterface.h"

#include <d3dx8.h>
#include <d3d8.h>
#include <map>

/**
* This object is used to manage and store our Textures
* for our small DLL rendering system for our Direct3D8 implementation.
* For now it doesn't really do much besides load and
* create texture objects, but the flexibility is 
* there to add more features if needed
*/
class D3DTextureManager : public textureManagerInterface {

protected:
	LPDIRECT3DDEVICE8                 m_lpD3DDevice; /** our Direct3DDevice interface */
	std::map<int, LPDIRECT3DTEXTURE8> m_oTextures;   /** map object containing texture data */


public:
	/**
	* Constructor
	* @param LPDIRECT3DDEVICE8 - our valid Direct3DDevice8 interface
	*/
	D3DTextureManager(LPDIRECT3DDEVICE8);

	/**
	* Destructor
	*/
	~D3DTextureManager();

	/**
	* version 1.3
	* This method is responsible for adding a new texture to our
	* rendering engine. We won't do anything that fancy but store
	* it in a std::map object
	* @param std::string - our texture filename
	* @param const int   - the texture identifier
	* @return HRESULT    - SUCCEEDED if we created it ok, FAILED otherwise
	*/
	HRESULT addTexture(std::string strFilename, const int value);

	/**
	* version 1.3
	* This method is responsible for letting the renderer engine
	* know which texture we want to use
	* @param const int - the texture identifier
	* @return void
	*/
	void setTexture(const int value);



};

#endif