
/**************************************************************************************
* This file is part of the GameFramework project released by Wazoo Enterprises Inc.
* under the GNU Public License. 
*
* Any projects using this code must attach these copyright notices along with their
* own as outlined at http://www.gnu.org
*
* Copyright 2003 by Wazoo Enterprises Inc.
* To contact the auther, send a message to wazoo@wazooenterprises.com
*
****************************************************************************************/

#include "D3DVertexBuffer.h"


D3DVertexBuffer::D3DVertexBuffer() : vbInterface(){

	
	m_lpVB = NULL;
	m_pVertices = NULL;
	m_lpD3DDevice = NULL;
	m_bIsLocked = FALSE;
	m_iVertexLimit = 0;


}

D3DVertexBuffer::~D3DVertexBuffer(){

	destroyVB();

}

HRESULT D3DVertexBuffer::createVB(int iNumOfTrianglesToBatch,
									   VOID *pDevice){

	HRESULT hr;

	if(m_bIsLocked){
		m_lpVB->Unlock();
		destroyVB();
	}
	
	m_iVertexLimit = iNumOfTrianglesToBatch*3;

	m_lpD3DDevice = (LPDIRECT3DDEVICE8)pDevice;

	if(FAILED(hr = m_lpD3DDevice->CreateVertexBuffer(
		m_iVertexLimit * sizeof(CUSTOM_VERTEX),
		D3DUSAGE_DYNAMIC|D3DUSAGE_WRITEONLY,
		D3DFVF_CUSTOM_VERTEX,
		D3DPOOL_DEFAULT,
		&m_lpVB))) {
		
		destroyVB();
		return E_FAIL;
	}

	return hr;

}

void D3DVertexBuffer::destroyVB(void){

	if(m_lpVB){
		m_lpVB->Release();
		m_lpVB = NULL;
	}

	m_bIsLocked = FALSE;
	m_iVerticesInBuffer = 0;



}


HRESULT D3DVertexBuffer::lockVB(void){

	HRESULT hr;

	m_iVerticesInBuffer=0;
	m_lpD3DDevice->SetStreamSource(0, m_lpVB, sizeof(CUSTOM_VERTEX));
	m_lpD3DDevice->SetVertexShader(D3DFVF_CUSTOM_VERTEX);
	//Lock the buffer for adding vertices
	hr = m_lpVB->Lock(0,0,(unsigned char**)&m_pVertices,
		D3DLOCK_DISCARD|D3DLOCK_NOOVERWRITE|D3DLOCK_NOSYSLOCK);

		
	m_bIsLocked = TRUE;

	return hr;

}

void D3DVertexBuffer::unlockVB(void){

	m_lpVB->Unlock();
	m_lpD3DDevice->DrawPrimitive(D3DPT_TRIANGLELIST,
		0,
		m_iVerticesInBuffer/3);
	
	m_bIsLocked = FALSE;

}


void D3DVertexBuffer::addTriToVB(D3DXVECTOR3 pos,
								 FLOAT r, FLOAT g, FLOAT b, FLOAT a){

	m_pVertices[m_iVerticesInBuffer].vecPos = pos;

	m_pVertices[m_iVerticesInBuffer].dwColor = D3DCOLOR_COLORVALUE(r, g, b, a);
	
	m_iVerticesInBuffer++;
	
	if(m_iVerticesInBuffer == m_iVertexLimit) {
		//Draw the now full buffer
		flushVB();
	}

}

void D3DVertexBuffer::flushVB(void){

	m_lpVB->Unlock();
	m_lpD3DDevice->DrawPrimitive(D3DPT_TRIANGLELIST,
		0,
		m_iVerticesInBuffer/3);	

	m_lpVB->Lock(0,0,(unsigned char**)&m_pVertices,
		D3DLOCK_DISCARD|D3DLOCK_NOOVERWRITE|D3DLOCK_NOSYSLOCK);

	
	m_iVerticesInBuffer=0;


}


void D3DVertexBuffer::addTriToVB(D3DXVECTOR3 pos,
								 FLOAT r, FLOAT g, FLOAT b, FLOAT a,
								 D3DXVECTOR2 tex){

	m_pVertices[m_iVerticesInBuffer].vecPos = pos;

	m_pVertices[m_iVerticesInBuffer].dwColor = D3DCOLOR_COLORVALUE(r, g, b, a);

	m_pVertices[m_iVerticesInBuffer].u = tex.x;
	m_pVertices[m_iVerticesInBuffer].v = -tex.y;
	
	m_iVerticesInBuffer++;
	
	if(m_iVerticesInBuffer == m_iVertexLimit) {
		//Draw the now full buffer
		flushVB();
	}

}