
#ifndef __OGLRENDERER_H_
#define __OGLRENDERER_H_

/**************************************************************************************
* This file is part of the GameFramework project released by Wazoo Enterprises Inc.
* under the GNU Public License. 
*
* Any projects using this code must attach these copyright notices along with their
* own as outlined at http://www.gnu.org
*
* Copyright 2003 by Wazoo Enterprises Inc.
* To contact the auther, send a message to wazoo@wazooenterprises.com
*
****************************************************************************************/

#include "rendererInterface.h"
#include <GL/gl.h>
#include <GL/glu.h>


/**
* This is our "low level" implementation of
* our rendererInterface class in OpenGL
* Note that it's VERY basic, but just meant to
* show a POSSIBLE implementation
*/
class OGLRenderer : public rendererInterface
{
private:
	HDC	  m_hDC;         /** handle to our main device context */
	HGLRC m_hRC;         /** handle to our OpenGL context      */
	
	GLenum	 m_srcBlendOp;
	GLenum   m_destBlendOp;

public:
	/**
	* Constructor
	*/
	OGLRenderer();

	/**
	* Destructor
	*/
	~OGLRenderer();

	/**
	* Method to create and initialize our hardware
	* @param HWND - our window handle
	* @param DWORD - our window width
	* @param DWORD - our window height
	* @param DWORD - our window depth
	* @param BOOL  - flag for being windowed or fullscreen
	* @return HRESULT - SUCCEEDED if we're ok, FAILED otherwise
	*/
    HRESULT createDevice(HWND, DWORD, DWORD, DWORD, BOOL);

    /**
	* Method to destroy it
	* @param void
	* @return void
	*/
    void destroyDevice(void);

    /**
	* Method to begin rendering primitives
	* @param void
	* @return HRESULT - SUCCEEDED if our hardware pipeline is ready, FAILED if not
	*/
    HRESULT beginRenderingScene(void);

    /**
	* Method to end rendering primitives
	* @param void
	* @return void
	*/
    void endRenderingScene(void);

    /**
	* Method to change the screen clearing color
	* @param FLOAT - red component
	* @param FLOAT - green component
	* @param FLOAT - blue component
	* @param FLOAT - alpha component
	* @return void
	*/
    void setClearColor(FLOAT, FLOAT, FLOAT, FLOAT);

	/**
	* v1.4 
	* Method to enable/disable a state within the engine
	* @param eState - which state we're trying to modify
	* @param eOp     - the operation to perform on said state
	* @return void
	*/
	void setState(const eState, const eOp);

	void setFunction(const eFunction, const eCompareOp, FLOAT fRef);


};

#endif