
#ifndef __OGLTEXTUREMANAGER_H_
#define __OGLTEXTUREMANAGER_H_

/**************************************************************************************
* This file is part of the GameFramework project released by Wazoo Enterprises Inc.
* under the GNU Public License. 
*
* Any projects using this code must attach these copyright notices along with their
* own as outlined at http://www.gnu.org
*
* Copyright 2003 by Wazoo Enterprises Inc.
* To contact the auther, send a message to wazoo@wazooenterprises.com
*
****************************************************************************************/

#include "textureManagerInterface.h"

#include <gl\gl.h>
#include <gl\glu.h>
#include <map>

// These defines are used to tell us about the type of TARGA file it is
#define TGA_RGB		 2		// This tells us it's a normal RGB (really BGR) file
#define TGA_A		 3		// This tells us it's a ALPHA file
#define TGA_RLE		10		// This tells us that the targa is Run-Length Encoded (RLE)


struct sImage
{
	int channels;			// The channels in the image (3 = RGB : 4 = RGBA)
	int sizeX;				// The width of the image in pixels
	int sizeY;				// The height of the image in pixels
	UCHAR *data;	// The image pixel data

public:
	sImage(){};
	virtual ~sImage(){};
};

// This is our image structure for our targa data
struct sTGAImage : public sImage
{
};

struct sBMPImage : public sImage
{
	int stride;
	HBITMAP hBitmap;
};

/**
* This object is used to manage and store our OpenGL Textures
* implementation for our small DLL rendering system.
* For now it doesn't really do much besides load and
* create texture objects, but the flexibility is 
* there to add more features if needed
*/
class OGLTextureManager : public textureManagerInterface {

protected:
	std::map<int, GLuint> m_oTextures; /** (v1.3) map container to store texture data */

	sTGAImage*              loadTGA(std::string strFilename);
	sBMPImage*             loadBMP(std::string strFilename);

public:
	/**
	* Constructor
	*/
	OGLTextureManager();

	/**
	* Destructor
	*/
	~OGLTextureManager();

	/**
	* version 1.3
	* This method is responsible for adding a new texture to our
	* rendering engine. We won't do anything that fancy but store
	* it in a std::map object
	* @param std::string - our texture filename
	* @param const int   - the texture identifier
	* @return HRESULT    - SUCCEEDED if we created it ok, FAILED otherwise
	*/
	HRESULT addTexture(std::string strFilename, const int value);

	/**
	* version 1.3
	* This method is responsible for letting the renderer engine
	* know which texture we want to use
	* @param const int - the texture identifier
	* @return void
	*/
	void setTexture(const int value);


};

#endif