
#ifndef __RENDERERFACTORY_H_
#define __RENDERERFACTORY_H_
/**************************************************************************************
* This file is part of the GameFramework project released by Wazoo Enterprises Inc.
* under the GNU Public License. 
*
* Any projects using this code must attach these copyright notices along with their
* own as outlined at http://www.gnu.org
*
* Copyright 2003 by Wazoo Enterprises Inc.
* To contact the auther, send a message to wazoo@wazooenterprises.com
*
****************************************************************************************/


#include "rendererInterface.h"
#include "Logger.h"

/**
* 
* This Factory object is solely responsible for the
* proper creation of our rendererInterface object.
* Rather than clutter up our main game code with
* run-time DLL linking code, we can encapsulate all
* of that into this object. 
*
*/
class rendererFactory
{
private:
	rendererInterface	*m_pRenderer;  /** pointer to our rendererInterface   */

	HINSTANCE           m_hInst;       /** handle of our application instance */
   	HINSTANCE           m_hDLL;        /** handle of our DLL                  */

	

public:

	   /**
	   * Overloaded constructor which stores the application instance
	   * @param HINSTANCE - the handle to our application instance
	   */
   	  rendererFactory(HINSTANCE hInst);
   	  

	  /**
	  * Destructor
	  */
	  ~rendererFactory();

	     
	  /**
	  *Version1.1
	  *Modified the method to accept a HWND that we can modify the text to
	  * @param std::string - the type of renderer we want, OPENGL or D3D
	  * @param HWND        - the handle to our application window
	  * @return HRESULT    - if we SUCCEEDED or FAILED the operation
	  */
   	  HRESULT initInterface(std::string strType, HWND);

	  /**
      * This method just returns our renderer interface
	  * @param void
	  * @return rendererInterface - the pointer to our rendererInterface object
	  */
   	  rendererInterface* getInterface(void){ return m_pRenderer; };
   	  
	  /**
	  * This method is responsible for doing any cleanup of our 
	  * rendererInterface object
	  * @param void
	  * @return void
	  */
	  void destroyInterface(void);
};

#endif