
/**************************************************************************************
* This file is part of the GameFramework project released by Wazoo Enterprises Inc.
* under the GNU Public License. 
*
* Any projects using this code must attach these copyright notices along with their
* own as outlined at http://www.gnu.org
*
* Copyright 2003 by Wazoo Enterprises Inc.
* To contact the auther, send a message to wazoo@wazooenterprises.com
*
****************************************************************************************/

#include "rendererFactory.h"
#include "Logger.h"

rendererFactory::rendererFactory(HINSTANCE hInst)
{
	m_hInst = hInst;
	m_pRenderer = NULL;

	

}

rendererFactory::~rendererFactory()
{
	Logger::getInstance()->writeToLog("Attempting to release renderer interface");
	destroyInterface();
	
}

HRESULT rendererFactory::initInterface(std::string strIniFile, HWND hWnd)
{
	
	std::string strDLL;
	TCHAR strTemp[MAX_PATH];

	//just to be even more clever, we'll pass in an .ini
	//file to this Factory object, which will give us the 
	//added bonus of changing the renderer type on the fly,
	//rather than having to recompile each time..

	//we COULD create a class to encapsulate the Get/WritePrivateProfileString
	//behaviour, but for some clarity, I'll just stick to the
	//bare win32 calls...
	TCHAR currentPath[MAX_PATH];
	ZeroMemory(&currentPath, sizeof(MAX_PATH));
	GetCurrentDirectory(MAX_PATH, currentPath);
	strcat(currentPath, "\\");
	strcat(currentPath, strIniFile.c_str());

	DWORD dwRetVal = 0;
	TCHAR szType[MAX_PATH];
	DWORD dwSizeVal = MAX_PATH;
	dwRetVal = GetPrivateProfileString("Video",    // section name
									  "Renderer",       // key name
									  "OpenGL", // default string
									  szType,      // destination buffer
									  dwSizeVal,       // size of destination buffer
									  currentPath);   // initialization file name

	sprintf(strTemp, "Opening renderer: %s", szType);
	Logger::getInstance()->writeToLog(strTemp);

	if(strcmp(szType, "OpenGL") == 0)
	{
		
		strDLL = "GameOGLRenderer";
	
	}else{

		strDLL = "GameD3DRenderer";
	}


	m_hDLL = LoadLibraryEx(strDLL.c_str(),NULL,0);
	if(!m_hDLL)
	{
		sprintf(strTemp, "Error loading up %s", strDLL.c_str());
		Logger::getInstance()->writeToLog(strTemp);
			
		return E_FAIL;
	}

	sprintf(strTemp, "Successfully loaded: %s", strDLL.c_str());
	Logger::getInstance()->writeToLog(strTemp);


	//okay now that we've got our dll loaded, let's get a pointer
	//to our createInterface function
	CREATERENDERERINTERFACE CreateInterface = 0;
	HRESULT hr;

	CreateInterface = (CREATERENDERERINTERFACE)GetProcAddress(m_hDLL,
															  "createRendererInterface");


	if(NULL == CreateInterface)
	{
		sprintf(strTemp, "Error using GetProcAddress for CreateInterface");
		Logger::getInstance()->writeToLog(strTemp);

		return E_FAIL;
	}

	hr = CreateInterface(&m_pRenderer);
	if(FAILED(hr))
	{
		sprintf(strTemp, "Error using CreateInterface");
		Logger::getInstance()->writeToLog(strTemp);

		m_pRenderer = NULL;
		return E_FAIL;
	}

	return S_OK;
}

void rendererFactory::destroyInterface(void)
{
	DESTROYRENDERERINTERFACE DestroyInterface = 0;
	HRESULT hr;
	TCHAR strTemp[MAX_PATH];

	//function pointer to our destroyRendererInterface function
	//within our DLL
	DestroyInterface = (DESTROYRENDERERINTERFACE)GetProcAddress(m_hDLL,
																"destroyRendererInterface");
	//call the DLL's method to properly destroy our rendererInterface
	//object
/*
	if(NULL == DestroyInterface)
	{
		sprintf(strTemp, "Error using GetProcAddress for DestroyInterface");
		Logger::getInstance()->writeToLog(strTemp);
		m_pRenderer = NULL;

		Logger::getInstance()->writeToLog("Closing DLL");
		FreeLibrary(m_hDLL);

		return;
	}
*/

	hr = DestroyInterface(&m_pRenderer);
	if(FAILED(hr))
	{

		sprintf(strTemp, "Error using DestroyInterface");
		Logger::getInstance()->writeToLog(strTemp);

		m_pRenderer = NULL;
	}

	Logger::getInstance()->writeToLog("Closing DLL");

	//NOTE: Erik Yuzwa - January 05th, 2004 -
	//I have been suddenly getting a very annoying "access violation" error
	//whenever I call FreeLibrary when I'm using the OpenGL renderer interface
	//DLL. I suspect it's something to do with the compiler options I'm using
	//but I've hunted through the MSDN and nearly every programmer/game programmer
	//site I can find on the subject to no avail. For now, I've commented out
	//this code. YES it produces memory leaks but at least there's no obvious
	//program crashing.
	/*
	DWORD dwError = 0;
	BOOL bRet = FreeLibrary(m_hDLL);
	if(!bRet){
		dwError = GetLastError();
		TCHAR strError[MAX_PATH];
		sprintf(strError, "Error %d from unloading DLL", dwError);
		Logger::getInstance()->writeToLog(strError);
		OutputDebugString(strError);
	}
	*/


}